
local _, YssInstanceMaps = ...

local L = LibStub("AceLocale-3.0"):GetLocale("YssInstanceMaps", true)

if not YssInstanceMaps then
	_GYssInstanceMaps = _GYssInstanceMaps or {}
	YssInstanceMaps = _GYssInstanceMaps
end

local timerFrame = CreateFrame("Frame")
local animationGroup = timerFrame:CreateAnimationGroup()
local animation = animationGroup:CreateAnimation("Animation")
animation:SetDuration(6) -- increase this number if this Scanner starts disconnecting people without outside interference 
animationGroup:SetLooping("NONE")

local Scanner = CreateFrame("GameTooltip", "YssInstanceMaps_ScannerTooltip", nil, "GameTooltipTemplate")
Scanner:SetOwner(UIParent, "ANCHOR_NONE")

local function ItemFound(item)
	if YssInstanceMaps.queries[item] then
		if YssInstanceMaps.queries[item].type then
			YssInstanceMaps.queries[item]:UpdateItem()
		else
			for i, btn in ipairs(YssInstanceMaps.queries[item]) do
				btn:UpdateItem()
			end
		end
		YssInstanceMaps.queries[item] = nil
	end
end

local itemlist = {}
local query_item
local function QueryNextItem()
	if next(itemlist) then
		query_item = tremove(itemlist, 1) -- always pull from bottom of table (its expensive but ensures order also makes adding items inexpensive and spreads out the expensive operation)
		if GetItemInfo(query_item) then -- if we already know item no need to query for it we just move on to the next item on the list
			ItemFound(query_item)
			return QueryNextItem()
		end
		Scanner:SetHyperlink('item:'..query_item)
	end
end

local function OnTooltipSetItem()
	if GetItemInfo(query_item) then -- if we already know item no need to query for it we just move on to the next item on the list
		ItemFound(query_item)
		if animationGroup:IsPlaying() then
			animationGroup:Stop()
		end
		QueryNextItem()
	else
		animationGroup:Play()
	end
end

animationGroup:SetScript("OnFinished", QueryNextItem)
Scanner:SetScript('OnTooltipSetItem', OnTooltipSetItem)

function YssInstanceMaps:QueryItemInfo(item)
	if GetItemInfo(item) then -- do a preliminary check if we already know the item
		return ItemFound(query_item)
	end
	for i, listitem in ipairs(itemlist) do -- check if this item is already in our list
		if item == listitem then
			return nil
		end
	end
	itemlist[#itemlist+1] = item
	if not query_item then
		QueryNextItem()
	end
end

function YssInstanceMaps:CancelAllQueries()
	wipe(itemlist)
	query_item = nil
	if YssInstanceMaps.queries then
		wipe(YssInstanceMaps.queries)
	end
	animationGroup:Stop()
end

--/script YssInstanceMapsTest()
function YssInstanceMapsTest()
	for i=1,10000 do
		YssInstanceMaps:QueryItemInfo(i)
	end
end